@@ -18,16 +18,16 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include <qapplication.h>
+#include <QApplication>
 #include <QMouseEvent>
-#include <qmessagebox.h>
-#include <qmenu.h>
-#include <qclipboard.h>
-#include <qtextedit.h>
-#include <qpainter.h>
-#include <qcursor.h>
-#include <qimage.h>
-//#include <QLocale>
+#include <QMessageBox>
+#include <QMenu>
+#include <QClipboard>
+#include <QTextEdit>
+#include <QPainter>
+#include <QCursor>
+#include <QImage>
+#include <QFileInfo>
 #include <stdio.h>
 #include <mgl/mgl_parse.h>
 #include <mgl/mgl_idtf.h>
@@ -43,7 +43,7 @@ QMGLCanvas::QMGLCanvas(QWidget *parent, Qt::WindowFlags f) : QWidget(parent, f)
 	popup = 0;		grBuf = 0;
 	phi = tet = per = 0;
 	x1 = y1 = 0;	x2 = y2 = 1;
-	alpha = light = zoom = rotate = false;
+	alpha = light = zoom = rotate = grid = false;
 	resize(graph->GetWidth(), graph->GetHeight());
 	convertFromGraph(pic, graph, &grBuf);
 	thr = new MGLThread();	thr->cnv = this;
@@ -58,6 +58,18 @@ void QMGLCanvas::paintEvent(QPaintEvent *)
 	paint.begin(this);
 	paint.drawPixmap(0,0,pic);
 	if(zoom)	paint.drawRect(x0,y0,xe-x0,ye-y0);
+	else if(grid)
+	{
+		int i, h=pic.height(), w=pic.width();
+		paint.setPen(QColor(192,192,192));
+		for(i=1;i<10;i++)
+		{
+			paint.drawText(0,i*h/10,QString::number(1-i*0.1));
+			paint.drawLine(0,i*h/10,w,i*h/10);
+			paint.drawText(i*w/10,h,QString::number(i*0.1));
+			paint.drawLine(i*w/10,0,i*w/10,h);
+		}
+	}
 	paint.end();
 }
 //-----------------------------------------------------------------------------
@@ -74,12 +86,17 @@ void QMGLCanvas::setPer(int p)
 	{	per = 100*p;	emit perChanged(p);	execute();	}
 }
 //-----------------------------------------------------------------------------
+void QMGLCanvas::imgSize(int w, int h)	{	graph->SetSize(w,h);	execute();	}
+//-----------------------------------------------------------------------------
 void QMGLCanvas::setPhi(int p)
 {	if(phi!=p)	{	phi = p;	emit phiChanged(p);	execute();	}	}
 //-----------------------------------------------------------------------------
 void QMGLCanvas::setTet(int t)
 {	if(tet!=t)	{	tet = t;	emit tetChanged(t);	execute();	}	}
 //-----------------------------------------------------------------------------
+void QMGLCanvas::setGrid(bool a)
+{	if(grid!=a)		{	grid = a;	emit gridChanged(a);	execute();	}	}
+//-----------------------------------------------------------------------------
 void QMGLCanvas::setAlpha(bool a)
 {	if(alpha!=a)	{	alpha = a;	emit alphaChanged(a);	execute();	}	}
 //-----------------------------------------------------------------------------
@@ -102,7 +119,8 @@ void QMGLCanvas::setRotate(bool r)
 	{
 		zoom=false;	rotate=r;
 		emit zoomChanged(false);	emit rotateChanged(r);
-		execute();	}
+		execute();
+	}
 }
 //-----------------------------------------------------------------------------
 void QMGLCanvas::shiftDown()
@@ -141,7 +159,7 @@ void QMGLCanvas::zoomOut()
 	execute();
 }
 //-----------------------------------------------------------------------------
-void QMGLCanvas::execute(mglGraph *gr)
+void QMGLCanvas::execute(mglGraphAB *gr)
 {
 	if(!textMGL && autoResize)	{	run(defText, false, gr);	return;	}
 	else if(!textMGL)	return;
@@ -150,7 +168,7 @@ void QMGLCanvas::execute(mglGraph *gr)
 	if(pic.size()!=size())	setSize(pic.width(), pic.height());
 }
 //-----------------------------------------------------------------------------
-QString QMGLCanvas::run(const QString &text, bool showm, mglGraph *gr)
+QString QMGLCanvas::run(const QString &text, bool showm, mglGraphAB *gr)
 {
 	defText = text;
 	if(gr==0)	gr=graph;
@@ -196,7 +214,7 @@ void MGLThread::run()
 	{
 		cur = text.section('\n',i,i);
 		r = cur.toWCharArray(str);	str[r] = 0;
-		gr->Message[0] = 0;			r = 0;
+		gr->Message[0] = 0;			gr->ObjId = i+1;
 		setlocale(LC_ALL, "C");
 		r = parser.Parse(gr,str,i+1);
 		setlocale(LC_ALL, "");
@@ -210,28 +228,33 @@ void MGLThread::run()
 			if(r==4)	mes += tr(" -- unbalanced ' :\t");
 			mes += cur;
 			warn += mes + "\n";
-//			cnv->showMess(mes);
 		}
 		if(gr->Message && gr->Message[0]!=0)
 		{
-			mes = tr("In line ")+QString::number(i)+tr(" -- warning:\t");
+			mes = tr("In line ")+QString::number(i)+tr(" :\t");
 			mes = mes + cur + "\n" + QString(gr->Message) + "\n";
 			warn += mes + "\n";
-//			cnv->showMess(mes);
 		}
 	}
 	mes = QString(gr->Message);
 	gr->Message=0;	delete []buf;	delete []str;
 }
 //-----------------------------------------------------------------------------
+void QMGLCanvas::mouseDoubleClickEvent(QMouseEvent *ev)
+{
+	int oi = graph->OI[ev->x()+graph->GetWidth()*ev->y()]-1;
+	emit objChanged(oi);
+}
+//-----------------------------------------------------------------------------
 void QMGLCanvas::mousePressEvent(QMouseEvent *ev)
 {
 	if(!zoom && !rotate && ev->button()&Qt::LeftButton)
 	{
 		mglPoint p = graph->CalcXYZ(ev->x(), ev->y());
-		QString str;
-		str.sprintf("x=%g, y=%g, z=%g",p.x,p.y,p.z);
-		emit posChanged(str);
+		clickPos.sprintf("x=%g, y=%g, z=%g  --  Xabs=%g, Yabs=%g",
+					p.x,p.y,p.z,double(ev->x())/graph->GetWidth(),
+					1-double(ev->y())/graph->GetHeight());
+		emit posChanged(clickPos);
 	}
 	xe=x0=ev->x();	ye=y0=ev->y();
 	ev->accept();
@@ -308,6 +331,18 @@ QString setExtension(QString &fname, const char *ext)
 	return oname;
 }
 //-----------------------------------------------------------------------------
+void QMGLCanvas::startGIF(int animDelay, QString fname)
+{	if(fname.isEmpty())	fname = scriptName;
+	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename. Please save script first."),QMessageBox::Ok,0,0);
+	else
+	{
+		graph->StartGIF(setExtension(fname,"gif").toAscii(), animDelay);
+		graph->ResetFrames();
+	}
+}
+//-----------------------------------------------------------------------------
+void QMGLCanvas::closeGIF()		{	graph->CloseGIF();	}
+//-----------------------------------------------------------------------------
 void QMGLCanvas::exportGIF(QString fname)
 {	if(fname.isEmpty())	fname = scriptName;
 	if(fname.isEmpty())	QMessageBox::critical(this, appName, tr("No filename. Please save script first."),QMessageBox::Ok,0,0);
@@ -452,17 +487,22 @@ void convertFromGraph(QPixmap &pic, mglGraphAB *gr, uchar **buf)
 }
 //-----------------------------------------------------------------------------
 void QMGLCanvas::copy()
-{
-	QClipboard *cb = QApplication::clipboard();
-	cb->setPixmap(pic, QClipboard::Clipboard);
-}
+{	QApplication::clipboard()->setPixmap(pic, QClipboard::Clipboard);	}
+//-----------------------------------------------------------------------------
+void QMGLCanvas::copyClickCoor()
+{	QApplication::clipboard()->setText(clickPos);	}
 //-----------------------------------------------------------------------------
 void QMGLCanvas::setMGLFont(QString path)
 {
 	if(path.isEmpty())	graph->GetFont()->Restore();
-	else	graph->GetFont()->Load(path.toAscii());
+	else
+	{
+		QFileInfo fi(path);
+		graph->GetFont()->Load(fi.baseName().toAscii(), fi.path().toAscii());
+	}
+	execute();
 }
 //-----------------------------------------------------------------------------
 void QMGLCanvas::setSize(int w, int h)
-{	graph->SetSize(w,h);	resize(w, h);	execute();	};
+{	graph->SetSize(w,h);	resize(w, h);	execute();	}
 //-----------------------------------------------------------------------------
