@@ -17,21 +17,20 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#include <QTime>
-#include <qtextedit.h>
+#include <QTextEdit>
 #include <QPrintDialog>
 #include <QCloseEvent>
-#include <qstatusbar.h>
-#include <qmessagebox.h>
+#include <QStatusBar>
+#include <QMessageBox>
 #include <QPrinter>
-#include <qapplication.h>
-#include <qpainter.h>
-#include <QScrollArea>
-#include <qsplitter.h>
-#include <qtimer.h>
-#include <qaction.h>
-#include <qpixmap.h>
-#include <QTabWidget>
+#include <QApplication>
+#include <QPainter>
+#include <QSplitter>
+#include <QAction>
+#include <QPixmap>
+#include <QMdiArea>
+#include <QMdiSubWindow>
+#include <QDockWidget>
 #include <mgl/mgl_parse.h>
 //-----------------------------------------------------------------------------
 #include "scriptwindow.h"
@@ -41,18 +40,16 @@
 #include "helpwindow.h"
 #include "optiondialog.h"
 #include "styledialog.h"
-#include "animparam.h"
-#include "argsdialog.h"
 #include "propdialog.h"
 #include "setupdialog.h"
 #include "infodialog.h"
-//#include "xpm/udav.xpm"
+#include "graphwindow.h"
+#include "infodialog.h"
+#include "newcmddialog.h"
 //-----------------------------------------------------------------------------
 extern bool mglAutoExecute;
 extern bool mglAutoSave;
-extern HelpWindow *hlp;
 extern mglParse parser;
-int animDelay=500;
 ArgsDialog *args_dlg=0;
 QString defFontFamily;
 int defFontSize;
@@ -61,65 +58,77 @@ int ScriptWindow::num_wnd=0;
 //-----------------------------------------------------------------------------
 ScriptWindow::ScriptWindow(QWidget *wp) : QMainWindow(wp)
 {
-	gifOn = jpgOn = false;
 	setWindowTitle(tr("untitled - UDAV"));
 	setAttribute(Qt::WA_DeleteOnClose);
 	printer = new QPrinter;
 	if(!args_dlg)	args_dlg = new ArgsDialog;
-	infoDlg = new InfoDialog(this);
-	infoDlg->setModal(true);	infoDlg->allowRefresh=false;
 	findDialog = new FindDialog(this);
 	optDialog = new OptionDialog(this);
 	stlDialog = new StyleDialog(this);
-	animDialog = new AnimParam(this);
 	setupDlg = new SetupDialog(this);
+	dataOpenDlg = new DataOpenDialog(this);
+	newCmdDlg = new NewCmdDialog(this);
+
 	connect(findDialog, SIGNAL(findText(const QString &, bool, bool)), this, SLOT(findText(const QString &, bool, bool)));
 	connect(findDialog, SIGNAL(replText(const QString &, const QString &, bool, bool)), this, SLOT(replText(const QString &, const QString &, bool, bool)));
-	connect(animDialog, SIGNAL(putText(const QString &)), this, SLOT(animPutText(const QString &)));
 	connect(setupDlg, SIGNAL(putText(const QString &)), this, SLOT(animPutText(const QString &)));
 
-	animPos = -1;
-	timer = new QTimer(this);
-	connect(timer, SIGNAL(timeout()), this, SLOT(nextSlide()));
-
-	splitG = new QSplitter(this);	splitG->setOrientation(Qt::Vertical);
-	split = new QSplitter(splitG);
-	edit = new QTextEdit(split);
-	edit->setFocus();	edit->setAcceptRichText(false);
+	split = new QSplitter(this);
+	edit = new QTextEdit(split);	edit->setAcceptRichText(false);
 	connect(edit, SIGNAL(cursorPositionChanged()), this, SLOT(editPosChanged()));
 	new QMGLSyntax(edit);
 	defFontFamily = edit->fontFamily();
-	defFontSize = edit->fontPointSize();
+	defFontSize = int(edit->fontPointSize());
 
+	info = new MemoryWindow(this);
+	graph = new GraphWindow(this);	graph->mgl->textMGL = edit;
+	connect(graph, SIGNAL(save()), this, SLOT(save()));
+	connect(graph, SIGNAL(statusBarMessage(const QString &)), this, SLOT(statusBarMessage(const QString &)));
+	connect(graph, SIGNAL(animPutText(const QString &)), this, SLOT(animPutText(const QString &)));
+	connect(graph->mgl, SIGNAL(objChanged(int)), this, SLOT(setCursorPosition(int)));
+	connect(graph->mgl, SIGNAL(posChanged(QString)), statusBar(), SLOT(showMessage(QString)));
+	connect(graph->mgl, SIGNAL(refreshData()), info, SLOT(refresh()));
 
-	sv = new QScrollArea(split);
-	mgl = new QMGLCanvas(split);	mgl->textMGL = edit;
-//	connect(this, SIGNAL(gotoLine(int,int)), this, SLOT(setCursorPosition(int,int)));
-	connect(mgl, SIGNAL(posChanged(QString)), statusBar(), SLOT(showMessage(QString)));
-	connect(mgl, SIGNAL(refreshData()), this, SLOT(refresh()));
-	sv->setWidget(mgl);
+	mdi = new QMdiArea(split);
+	mdi->addSubWindow(graph);	graph->setWindowState(Qt::WindowMaximized);
+	mdi->addSubWindow(info);	info->setWindowState(Qt::WindowMaximized);
+	hlp = new HelpWindow(this);	mdi->addSubWindow(hlp);
+	hlp->setWindowState(Qt::WindowMaximized);
+	mdi->setActiveSubWindow(graph);	mdi->setViewMode(QMdiArea::TabbedView);
+	connect(mdi, SIGNAL(subWindowActivated(QMdiSubWindow *)), this, SLOT(subActivated(QMdiSubWindow *)));
 
-	info = setupInfo(splitG);	mgl->warnMGL = mess;	new MessSyntax(mess);
+	messWnd = new QDockWidget(tr("Messages and warnings"),this);
+	mess = new QTextEdit(this);	messWnd->setWidget(mess);
+	messWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+	addDockWidget(Qt::BottomDockWidgetArea, messWnd);
+	messWnd->resize(size().width(), 0);
+	graph->mgl->warnMGL = mess;	new MessSyntax(mess);
 	connect(mess, SIGNAL(textChanged()), this, SLOT(warnChanged()));
 
+	calcWnd = new QDockWidget(tr("Calculator"),this);
+	calc = new CalcDialog();	calcWnd->setWidget(calc);
+	calcWnd->setAllowedAreas(Qt::TopDockWidgetArea | Qt::BottomDockWidgetArea);
+	addDockWidget(Qt::BottomDockWidgetArea, calcWnd);
+	calcWnd->resize(size().width(), 200);
+	connect(calc, SIGNAL(putNumber(QString)),this,SLOT(putText(QString)));
+
 	makeMenu();
-//	setDockMenuEnabled(true);
-	mgl->setPopup(popup);
-	setCentralWidget(splitG);
+	setCentralWidget(split);
 	setWindowIcon(QIcon(":/udav.png"));
 	readSettings();
 	if(!propDlg)	propDlg = new PropDialog;
 	edit->setLineWrapMode(QTextEdit::NoWrap);
+	connect(propDlg, SIGNAL(sizeChanged(int,int)), graph->mgl, SLOT(imgSize(int,int)));
 	connect(edit,SIGNAL(textChanged()), this, SLOT(setAsterix()));
-	info->setCurrentIndex(1);
+	edit->setFocus();	// TODO: not work right now :(
 	statusBar()->showMessage(tr("Ready"), 2000);
 	num_wnd++;
 }
 //-----------------------------------------------------------------------------
-ScriptWindow::~ScriptWindow()
-{
-	delete printer;
-}
+ScriptWindow::~ScriptWindow()	{	delete printer;	}
+//-----------------------------------------------------------------------------
+void ScriptWindow::statusBarMessage(const QString &txt)
+{	statusBar()->showMessage(txt, 2000);	}
 //-----------------------------------------------------------------------------
 void ScriptWindow::printText()
 {
@@ -134,39 +143,14 @@ void ScriptWindow::printText()
 		statusBar()->showMessage(tr("Printing aborted"), 2000);
 }
 //-----------------------------------------------------------------------------
-void ScriptWindow::printPlot()
-{
-	QPrintDialog printDlg(printer, this);
-	if (printDlg.exec() == QDialog::Accepted)
-	{
-		statusBar()->showMessage(tr("Printing..."));
-		QPainter p;
-		if(!p.begin(printer))	return;	// paint on printer
-		QRect r = p.viewport();
-		int w = r.width(), h = r.height(), h1;
-		h1 = int(w/mgl->getRatio());
-		if(h1<h)	h = h1;	else	w = int(h*mgl->getRatio());
-		mglGraphZB gr(w, h);
-		if(w*h > 240000)	gr.BaseLineWidth = sqrt(w*h/2.4e5);
-		mgl->execute(&gr);
-
-		uchar *grBuf=0;
-		QPixmap pic;
-		convertFromGraph(pic, &gr, &grBuf);
-		p.drawPixmap(0,0,pic);
-		delete []grBuf;
-		statusBar()->showMessage(tr("Printing completed"), 2000);
-	}
-	else
-		statusBar()->showMessage(tr("Printing aborted"), 2000);
-}
-//-----------------------------------------------------------------------------
 void ScriptWindow::closeEvent(QCloseEvent* ce)
 {
 	bool ok=true;
 	writeSettings();
 	if(edit->document()->isModified())
-		switch(QMessageBox::information(this, tr("Qt Application Example"), tr("Do you want to save the changes to the document?"), QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel))
+		switch(QMessageBox::information(this, tr("UDAV"),
+				tr("Do you want to save the changes to the document?"),
+				QMessageBox::Yes, QMessageBox::No, QMessageBox::Cancel))
 		{
 			case QMessageBox::Yes:	save();	break;
 			case QMessageBox::No:	break;
@@ -181,15 +165,6 @@ void ScriptWindow::closeEvent(QCloseEvent* ce)
 	else	ce->ignore();
 }
 //-----------------------------------------------------------------------------
-void ScriptWindow::pressF5()
-{
-	if(mglAutoSave)	save();
-	QTime t;
-	t.start();
-	mgl->execute();
-	statusBar()->showMessage(QString(tr("Drawing time %1 ms")).arg(t.elapsed()*1e-3));
-}
-//-----------------------------------------------------------------------------
 void ScriptWindow::find()
 {
 	findDialog->show();
@@ -225,7 +200,7 @@ void ScriptWindow::replText(const QString &str, const QString &txt, bool cs, boo
 //-----------------------------------------------------------------------------
 void ScriptWindow::showHelp()
 {
-	QString s = edit->textCursor().block().text(), dlm(" #;\t");
+	QString s = edit->textCursor().block().text(), dlm(" #;:\t");
 	int i, n = s.length();
 	for(i=0;i<n;i++)	if(dlm.contains(s[i]))	break;
 	s.truncate(i);
@@ -237,7 +212,7 @@ int mgl_cmd_cmp(const void *a, const void *b);
 void ScriptWindow::editPosChanged()
 {
 	register int i, n, m;
-	QString text = edit->textCursor().block().text(), dlm(" #;\t");
+	QString text = edit->textCursor().block().text(), dlm(" #;:\t");
 	n = text.length();
 	for(i=0;i<n;i++)	if(dlm.contains(text[i]))	break;
 	text.truncate(i);	m = text.length();
@@ -264,126 +239,13 @@ void ScriptWindow::addOptions()
 	}
 }
 //-----------------------------------------------------------------------------
-void ScriptWindow::animStart(bool st)
-{
-	if(animParam.isEmpty())
-	{
-		if(animDialog->exec())
-		{
-			animParam = animDialog->getResult();
-			gifName = animDialog->gifName;
-			gifOn = animDialog->gifOn;
-			jpgOn = animDialog->jpgOn;
-		}
-		else	return;
-	}
-	if(st)
-	{
-		timer->start(animDelay);
-		if(gifOn)	mgl->graph->StartGIF(gifName.toAscii(), animDelay);
-		mgl->graph->ResetFrames();
-	}
-	else
-	{
-		timer->stop();
-		if(gifOn)	mgl->graph->CloseGIF();
-	}
-}
-//-----------------------------------------------------------------------------
-void ScriptWindow::nextSlide()
-{
-	if(animParam.isEmpty())
-	{
-		if(animDialog->exec())
-		{
-			animParam = animDialog->getResult();
-			gifName = animDialog->gifName;
-			gifOn = animDialog->gifOn;
-			jpgOn = animDialog->jpgOn;
-		}
-		else	return;
-	}
-	int l=animParam.length(), n=animParam.count('\n') + (animParam[l-1]=='\n' ? 0:1), i;
-	wchar_t *str = new wchar_t[l+2];
-	animPos = (animPos+1)%n;
-	QString cur = animParam.section('\n',animPos,animPos);
-	for(i=0;i<l;i++)	str[i] = (cur[i]).unicode();
-	str[i] = 0;
-	parser.AddParam(0,str);
-	if(mgl->graph->GetNumFrame() >= n)
-		mgl->execute();
-	else
-	{
-		mgl->graph->NewFrame();
-		mgl->execute();
-		mgl->graph->EndFrame();
-		if(jpgOn)	mgl->graph->WriteFrame();
-		QString s;	s.sprintf("%d - %d of %d",mgl->graph->GetNumFrame(),animPos,n);
-		statusBar()->showMessage(QString(tr("Frame %1 of %2")).arg(animPos).arg(n));
-	}
-}
-//-----------------------------------------------------------------------------
-void ScriptWindow::prevSlide()
-{
-	if(animParam.isEmpty())
-	{
-		if(animDialog->exec())
-		{
-			animParam = animDialog->getResult();
-			gifName = animDialog->gifName;
-			gifOn = animDialog->gifOn;
-			jpgOn = animDialog->jpgOn;
-		}
-		else	return;
-	}
-	int l=animParam.length(), n=animParam.count('\n') + (animParam[l-1]=='\n' ? 0:1), i;
-	wchar_t *str = new wchar_t[l+2];
-	animPos = (animPos-1+n)%n;
-	QString cur = animParam.section('\n',animPos,animPos);
-	for(i=0;i<l;i++)	str[i] = (cur[i]).unicode();
-	str[i] = 0;
-	parser.AddParam(0,str);
-	mgl->execute();
-}
-//-----------------------------------------------------------------------------
-void ScriptWindow::animSetup()
-{
-	if(animDialog->exec())
-	{
-		animParam = animDialog->getResult();
-		gifName = animDialog->gifName;
-		gifOn = animDialog->gifOn;
-		jpgOn = animDialog->jpgOn;
-	}
-}
-//-----------------------------------------------------------------------------
-void ScriptWindow::addSetup()
-{
-	setupDlg->exec();
-}
+void ScriptWindow::addSetup()	{	setupDlg->exec();	}
 //-----------------------------------------------------------------------------
 void ScriptWindow::animPutText(const QString &s)
-{
-	edit->moveCursor(QTextCursor::Start);
-	edit->insertPlainText(s);
-}
+{	edit->moveCursor(QTextCursor::Start);	edit->insertPlainText(s);	}
 //-----------------------------------------------------------------------------
-void ScriptWindow::animParseText()
-{
-	int i, n = edit->toPlainText().count('\n')+1;
-	QString s, all;
-	for(i=0;i<n;i++)
-	{
-		s = edit->toPlainText().section('\n',i,i);
-		if(s[0]=='#' && s[1]=='#' && s[2]=='a' && (s[3]==' ' || s[3]=='\t'))
-			all = all + s.mid(4) + "\n";
-	}
-	if(!all.isEmpty())
-	{
-		animDialog->setResult(all);
-		animParam = all;
-	}
-}
+void ScriptWindow::putText(const QString &txt)
+{	edit->insertPlainText(txt);	}
 //-----------------------------------------------------------------------------
 void ScriptWindow::addStyle()
 {
@@ -405,29 +267,23 @@ void ScriptWindow::addStyle()
 void ScriptWindow::setEditorFont(QFont *f)
 {	edit->setFont(f ? *f : QFont(defFontFamily, defFontSize));	}
 //-----------------------------------------------------------------------------
-void ScriptWindow::setMGLFont(QString path)	{	mgl->setMGLFont(path);	}
-//-----------------------------------------------------------------------------
 void ScriptWindow::setEditPos(bool bottom)
-{
-	splitG->setOrientation(bottom? Qt::Horizontal : Qt::Vertical);
-	split->setOrientation(bottom ? Qt::Vertical : Qt::Horizontal);
-}
+{	split->setOrientation(bottom ? Qt::Vertical : Qt::Horizontal);	}
 //-----------------------------------------------------------------------------
 void ScriptWindow::properties()	{	propDlg->exec();	}
 //-----------------------------------------------------------------------------
-void ScriptWindow::setCursorPosition(int n,int)
+void ScriptWindow::setCursorPosition(int n)
 {
+	if(n<0)	return;
 	edit->moveCursor(QTextCursor::Start);
 	for(int i=0;i<n;i++)	edit->moveCursor(QTextCursor::NextBlock);
 	edit->setFocus();
 }
 //-----------------------------------------------------------------------------
-void ScriptWindow::adjust()	{	mgl->setSize(sv->width()-5, sv->height()-5);	}
-//-----------------------------------------------------------------------------
 void ScriptWindow::warnChanged()
 {
 	if(mess->toPlainText().isEmpty())	return;
-	info->setCurrentIndex(0);	ainfo->setChecked(true);
+	messWnd->show();	ainfo->setChecked(true);
 }
 //-----------------------------------------------------------------------------
 #ifndef USE_HDF5
@@ -462,11 +318,11 @@ void ScriptWindow::loadHDF5(const QString &fileName)
 			H5Dread(hd, H5T_C_S1, H5S_ALL, H5S_ALL, H5P_DEFAULT, buf);
 			buf[dims[0]]=0;		// to be sure :)
 			edit->setText(buf);
-			animParseText();
+			graph->animParseText(edit->toPlainText());
 			setCurrentFile(fileName);
 			delete []buf;
 			statusBar()->showMessage(tr("Loaded document %1").arg(fileName), 2000);
-			if(mglAutoExecute)	mgl->execute();
+			if(mglAutoExecute)	graph->mgl->execute();
 		}
 		else if(H5Tget_class(ht)==H5T_FLOAT || H5Tget_class(ht)==H5T_INTEGER)
 		{
@@ -499,9 +355,6 @@ void ScriptWindow::saveHDF5(const QString &fileName)
 	const char *fname = fileName.toAscii().constData();
 
 	H5Eset_auto(0,0);
-//	int res=H5Fis_hdf5(fname);
-//	if(res>0)	hf = H5Fopen(fname, H5F_ACC_RDWR, H5P_DEFAULT);
-//	else
 	hf = H5Fcreate(fname, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
 	if(hf<0)
 	{
