@@ -18,19 +18,18 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 #include <QTableWidget>
-#include <qlabel.h>
-#include <qaction.h>
-#include <qlayout.h>
-#include <qmenubar.h>
-#include <qlineedit.h>
-#include <qlineedit.h>
+#include <QLabel>
+#include <QAction>
+#include <QLayout>
+#include <QMenuBar>
+#include <QLineEdit>
 #include <QMenu>
-#include <qclipboard.h>
-#include <qstatusbar.h>
-#include <qfiledialog.h>
-#include <qpushbutton.h>
-#include <qapplication.h>
-#include <qinputdialog.h>
+#include <QClipboard>
+#include <QStatusBar>
+#include <QFileDialog>
+#include <QPushButton>
+#include <QApplication>
+#include <QInputDialog>
 #include <QToolBar>
 #include <QSpinBox>
 #include <mgl/mgl_parse.h>
@@ -42,52 +41,43 @@ extern mglParse parser;
 void updateDataItems();
 void deleteWindow(void *o)	{	if(o)	delete ((TableWindow *)o);	}
 //-----------------------------------------------------------------------------
-TableWindow::TableWindow(QWidget *parent) : QMainWindow(parent)
+TableWindow::TableWindow(InfoDialog *inf, QWidget *parent) : QMdiSubWindow(parent)
 {
 	setAttribute(Qt::WA_DeleteOnClose);
 	kz = nx = ny = nz = 0;	var = 0;
 	ready = false;
-
-	infoDlg = new InfoDialog(this);
+	infoDlg = inf;
 	tab = new QTableWidget(this);
 	connect(tab, SIGNAL(cellChanged(int,int)), this, SLOT(putValue(int, int)));
 
-	makeMenu();
-//	setDockMenuEnabled(true);
 	setWindowIcon(QPixmap(table_xpm));
-	setCentralWidget(tab);
-	QMainWindow::resize(700, 400);
-	statusBar()->showMessage(tr("Ready"), 2000);
+	setWidget(tab);
 }
 //-----------------------------------------------------------------------------
-TableWindow::~TableWindow()
-{
-	if(var)	var->o = 0;
-}
+TableWindow::~TableWindow()	{	if(var)	var->o = 0;	}
 //-----------------------------------------------------------------------------
 void TableWindow::refresh()
 {
 	bool rc = false;
 	if(!var)	return;
 	infoDlg->allowRefresh=false;
-	if(nx!=var->d.nx)
-	{	nx = var->d.nx;	tab->setColumnCount(nx);rc=true;	}
-	if(ny!=var->d.ny)
-	{	ny = var->d.ny;	tab->setRowCount(ny);	rc=true;	}
-	if(kz>=var->d.nz)
-	{	kz = 0;			emit sliceChanged(0);	}
-	if(nz!=var->d.ny)
-	{	nz = var->d.nz;	sb->setRange(0,nz-1);	emit nzChanged(nz);		}
+	if(nx!=var->d.nx)	{	nx = var->d.nx;	tab->setColumnCount(nx);rc=true;	}
+	if(ny!=var->d.ny)	{	ny = var->d.ny;	tab->setRowCount(ny);	rc=true;	}
+	if(kz>=var->d.nz)	{	kz = 0;			emit sliceChanged(0);	}
+	if(nz!=var->d.ny)	{	nz = var->d.nz;	emit nzChanged(nz);		}
 	register long i,j,m=wcslen(var->s);
 	register float f;
 	QString s,d;
 	if(rc)
 	{
-		QTableWidgetItem *it;
+		QStringList sh,sv;
+		for(i=0;i<nx;i++)	sh<<QString::number(i);
+		tab->setHorizontalHeaderLabels(sh);
+		for(i=0;i<ny;i++)	sv<<QString::number(i);
+		tab->setVerticalHeaderLabels(sv);
 		for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
-		{	it = new QTableWidgetItem;	tab->setItem(j,i,it);	}
+			tab->setItem(j,i,new QTableWidgetItem);
 	}
-//	tab->setNumCols(nx);	tab->setNumRows(ny);
 	for(i=0;i<nx;i++)	for(j=0;j<ny;j++)
 	{
 		f = var->d.a[i+nx*(j+ny*kz)];
@@ -101,7 +91,6 @@ void TableWindow::refresh()
 	s = QString(ss, m);	delete []ss;
 	d.sprintf("%d * %d * %d", nx, ny, nz);
 	ready = true;
-	statusBar()->showMessage(tr("Data: ") + s + tr(", sizes: ") + d);
 }
 //-----------------------------------------------------------------------------
 void TableWindow::setVar(mglVar *v)
@@ -149,228 +138,6 @@ void TableWindow::putValue(int r, int c)
 	infoDlg->refresh();
 }
 //-----------------------------------------------------------------------------
-#include "xpm/preview.xpm"
-#include "xpm/plot.xpm"
-#include "xpm/size.xpm"
-#include "xpm/smth.xpm"
-#include "xpm/oper_d.xpm"
-#include "xpm/oper_s.xpm"
-#include "xpm/oper_a.xpm"
-#include "xpm/oper_m.xpm"
-#include "xpm/crop.xpm"
-#include "xpm/tran.xpm"
-#include "xpm/integr.xpm"
-#include "xpm/diff.xpm"
-#include "xpm/diff2.xpm"
-#include "xpm/squize.xpm"
-#include "xpm/sum.xpm"
-#include "xpm/func.xpm"
-#include "xpm/swap.xpm"
-//-----------------------------------------------------------------------------
-void TableWindow::makeMenu()
-{
-	QAction *a;
-	QMenu *o, *oo;
-	QToolBar *bb;
-
-	// file menu
-	{
-	bb = new QToolBar(tr("File Operations"),this);
-	addToolBar(Qt::TopToolBarArea, bb);
-	o = menuBar()->addMenu(tr("&File"));
-	a = new QAction(QPixmap(":/xpm/document-open.png"), tr("&Load data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(load()));
-	a->setToolTip(tr("Load data from file (Ctrl+O). Data will be deleted only\nat exit but UDAV will not ask to save it."));
-	a->setShortcut(Qt::CTRL+Qt::Key_O);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/document-import.png"), tr("&Import PNG"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(imprt()));
-	a->setToolTip(tr("Import data from PNG picture with specified color scheme.\nData will be deleted only at exit but UDAV will not ask it saving."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/document-save.png"), tr("&Save data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(save()));
-	a->setToolTip(tr("Save data to tab-separeted file (Ctrl+S)."));
-	a->setShortcut(Qt::CTRL+Qt::Key_S);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/document-export.png"), tr("&Export PNG"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(exprt()));
-	a->setToolTip(tr("Export data from PNG picture. The colors defined by \nspecified color scheme. The same as in 'dens' command."));
-	o->addAction(a);	bb->addAction(a);
-	o->addSeparator();	bb->addSeparator();
-//	a = new QAction(QPixmap(insert_xpm), tr("Insert as &list"), this);
-//	connect(a, SIGNAL(activated()), this, SLOT(list()));
-//	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(plot_xpm), tr("Plot &data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(plot()));
-	a->setToolTip(tr("Plot data in new script window. You may select the kind\nof plot, its style and so on."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(preview_xpm), tr("Pre&view"), this);
-	a->setShortcut(Qt::Key_F2);	a->setCheckable(true);
-	connect(a, SIGNAL(toggled(bool)), infoDlg, SLOT(setVisible(bool)));
-	connect(infoDlg, SIGNAL(finished(int)), a, SLOT(toggle()));
-	a->setToolTip(tr("Open/close the preview/information window (F2).\nThis window show simple graphics and information about data."));
-	o->addAction(a);	bb->addAction(a);
-//	o->addSeparator();	bb->addSeparator();
-	a = new QAction(QPixmap(":/xpm/edit-copy.png"), tr("&Copy data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(copy()));
-	a->setToolTip(tr("Copy selected data to clipboard (Ctrl+C).\nValues are copied as tab-separated text."));
-	a->setShortcut(Qt::CTRL+Qt::Key_C);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/edit-paste.png"), tr("&Paste data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(paste()));
-	a->setToolTip(tr("Paste data from clipboard (Ctrl+V).\nIt is supposed that values have tab-separated text format."));
-	a->setShortcut(Qt::CTRL+Qt::Key_V);	o->addAction(a);	bb->addAction(a);
-	}
-	// size menu
-	{
-	bb = new QToolBar(tr("Resize data"),this);
-	addToolBar(Qt::TopToolBarArea, bb);
-	o = menuBar()->addMenu(tr("&Sizes"));
-	a = new QAction(QPixmap(":/xpm/document-new.png"), tr("&Create new"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(create()));
-	a->setToolTip(tr("Recreate the data with new sizes and fill it by zeros (Ctrl+N)."));
-	a->setShortcut(Qt::CTRL+Qt::Key_N);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(size_xpm), tr("&Resize"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(resize()));
-	a->setToolTip(tr("Resize (interpolate) the data to specified sizes (Ctrl+R)."));
-	a->setShortcut(Qt::CTRL+Qt::Key_R);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(squize_xpm), tr("&Squeeze"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(squize()));
-	a->setToolTip(tr("Keep only each n-th element of the data array."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(crop_xpm), tr("Cro&p"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(crop()));
-	a->setToolTip(tr("Crop the data edges. Useful to cut off the zero-filled area."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(tran_xpm), tr("&Transpose"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(transp()));
-	a->setToolTip(tr("Transpose data dimensions, like x<->y or x<->z and so on."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(tr("Re&arrange"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(rearrange()));
-	a->setToolTip(tr("Rearrange data sizes without changing data values."));
-	o->addAction(a);
-	}
-	// modify menu
-	{
-	bb = new QToolBar(tr("Modify data"),this);
-	addToolBar(Qt::LeftToolBarArea, bb);
-	o = menuBar()->addMenu(tr("&Modify"));
-
-	a = new QAction(QPixmap(func_xpm), tr("By &formula"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(byformula()));
-	a->setToolTip(tr("Change data values according to formula depended on 'x', 'y' and 'z'\nvariables (Ctrl+M). A set of special function is availible also."));
-	a->setShortcut(Qt::CTRL+Qt::Key_M);	o->addAction(a);	bb->addAction(a);
-	a = new QAction(tr("Fill in &range"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(inrange()));
-	a->setToolTip(tr("Fill data equidistantly from one value to another."));
-	o->addAction(a);
-	a = new QAction(tr("&Normalize"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(norm()));
-	a->setToolTip(tr("Normalize data so that its minimal\nand maximal values be in specified range."));
-	o->addAction(a);
-	a = new QAction(tr("Norm. s&lices"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(normsl()));
-	a->setToolTip(tr("Normalize each data slice perpendicular to some direction\nso that its minimal and maximal values be in specified range."));
-	o->addAction(a);
-	a = new QAction(QPixmap(smth_xpm), tr("&Smooth data"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(smooth()));
-	a->setToolTip(tr("Smooth data by one of 4 methods along specified direction(s)."));
-	o->addAction(a);	bb->addAction(a);
-	o->addSeparator();	bb->addSeparator();
-
-	oo = o->addMenu(tr("&Operators"));
-	a = new QAction(QPixmap(sum_xpm), tr("&Cum. sum"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(cumsum()));
-	a->setToolTip(tr("Summate data values along specified direction(s)."));
-	oo->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(integr_xpm), tr("&Integrate"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(integr()));
-	a->setToolTip(tr("Integrate data values along specified direction(s)."));
-	oo->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(diff_xpm), tr("&Differentiate"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(diff()));
-	a->setToolTip(tr("Differentiate data values along specified direction(s)."));
-	oo->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(diff2_xpm), tr("&Laplace"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(diff2()));
-	a->setToolTip(tr("Double differentiate data values along specified direction(s)."));
-	oo->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(swap_xpm), tr("&Swap"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(swap()));
-	a->setToolTip(tr("Swap left and right data part along specified direction(s).\nThis operation is useful for data after Fourier transform."));
-	oo->addAction(a);	bb->addAction(a);
-	a = new QAction(tr("&Mirror"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(mirror()));
-	a->setToolTip(tr("Mirror left and right data part along specified direction(s).\nThis operation do like index change from 'i' to 'n-i'."));
-	oo->addAction(a);
-
-	oo = o->addMenu(tr("&Algebraic"));
-	a = new QAction(QPixmap(oper_a_xpm), tr("&Add"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(addto()));
-	a->setToolTip(tr("Add a number to all data values."));
-	oo->addAction(a);
-	a = new QAction(QPixmap(oper_s_xpm), tr("&Subtract"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(subto()));
-	a->setToolTip(tr("Subtract a number to all data values."));
-	oo->addAction(a);
-	a = new QAction(QPixmap(oper_m_xpm), tr("&Multiply"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(multo()));
-	a->setToolTip(tr("Multiply all data values by a number."));
-	oo->addAction(a);
-	a = new QAction(QPixmap(oper_d_xpm), tr("&Divide"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(divto()));
-	a->setToolTip(tr("Divide all data values by a number."));
-	oo->addAction(a);
-
-	oo = o->addMenu(tr("A&nother data"));
-	a = new QAction(tr("&Sum of"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(sumof()));
-	a = new QAction(tr("M&in of"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(minof()));
-	a = new QAction(tr("M&ax of"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(maxof()));
-	a = new QAction(tr("Momentum along &x"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(momentx()));
-	a = new QAction(tr("Momentum along &y"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(momenty()));
-	a = new QAction(tr("Momentum along &z"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(momentz()));
-	a = new QAction(tr("&Histogram"), this);	oo->addAction(a);
-	connect(a, SIGNAL(activated()), this, SLOT(hist()));
-	}
-	// navigation menu
-	{
-	bb = new QToolBar(tr("Navigate data"),this);
-	addToolBar(Qt::TopToolBarArea, bb);
-	o = menuBar()->addMenu(tr("&Navigate"));
-	a = new QAction(QPixmap(":/xpm/go-first.png"), tr("&First slice"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(first()));
-	a->setToolTip(tr("Go to the first data slice for 3D data."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/go-previous.png"), tr("&Prev. slice"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(prev()));
-	a->setToolTip(tr("Go to the previous data slice for 3D data."));
-	o->addAction(a);	bb->addAction(a);
-
-	sb = new QSpinBox(this);
-	bb->addWidget(sb);	sb->setRange(0,0);
-	connect(sb, SIGNAL(valueChanged(int)), this, SLOT(setSlice(int)));
-	connect(this, SIGNAL(sliceChanged(int)), sb, SLOT(setValue(int)));
-	sb->setToolTip(tr("Go to the specified data slice for 3D data."));
-
-	a = new QAction(tr("Go to slice"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(gosl()));
-	a->setToolTip(tr("Go to the specified data slice for 3D data."));
-	o->addAction(a);
-	a = new QAction(QPixmap(":/xpm/go-next.png"), tr("Next slice"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(next()));
-	a->setToolTip(tr("Go to the next data slice for 3D data."));
-	o->addAction(a);	bb->addAction(a);
-	a = new QAction(QPixmap(":/xpm/go-last.png"), tr("Last slice"), this);
-	connect(a, SIGNAL(activated()), this, SLOT(last()));
-	a->setToolTip(tr("Go to the last data slice for 3D data."));
-	o->addAction(a);	bb->addAction(a);
-	}
-}
-//-----------------------------------------------------------------------------
 void TableWindow::imprt()
 {
 	QString fn = QFileDialog::getOpenFileName(this, tr("UDAV - Import PNG"), "", tr("Data files (*.dat)\nAll files (*.*)"));
@@ -609,7 +376,7 @@ void TableWindow::swap()
 void TableWindow::mirror()
 {
 	bool ok;
-	QString s = QInputDialog::getText(this, tr("UDAV - Mirror data"), tr("Enter direction(s) for mirroring.\nFor example 'xy' - mirror along x and y directions. Useful for Fourier spectrum."), QLineEdit::Normal, "", &ok);
+	QString s = QInputDialog::getText(this, tr("UDAV - Mirror data"), tr("Enter direction(s) for mirroring.\nFor example 'xy' - mirror along x and y directions."), QLineEdit::Normal, "", &ok);
 	if(ok)	{	var->d.Swap(s.toAscii());	refresh();	}
 }
 //-----------------------------------------------------------------------------
@@ -649,7 +416,8 @@ void TableWindow::minof()
 void TableWindow::momentx()
 {
 	QString name, val;
-	if(namesDialog(tr("UDAV - Momentum along 'x'"), tr("Specify which momentum evaluate.\nFor example, 'z2y' will give moment 'z^2*y'."), name, val))
+	if(namesDialog(tr("UDAV - Momentum along 'x'"),
+		tr("Specify which momentum evaluate.\nThe momentum is res_i = sum_jk how(x_i,y_j,z_k) a_jk/ sum_jk a_jk.\nCoordinates x, y, z are data indexes normalized in range [0,1]."), name, val))
 	{
 		mglVar *v = parser.AddVar(name.toAscii());
 		v->d = var->d.Momentum('x', val.toAscii());
@@ -660,7 +428,8 @@ void TableWindow::momentx()
 void TableWindow::momenty()
 {
 	QString name, val;
-	if(namesDialog(tr("UDAV - Momentum along 'y'"), tr("Specify which momentum evaluate.\nFor example, 'z2x' will give moment 'z^2*x'."), name, val))
+	if(namesDialog(tr("UDAV - Momentum along 'y'"),
+		tr("Specify which momentum evaluate.\nThe momentum is res_j = sum_ik how(x_i,y_j,z_k) a_ik/ sum_ik a_ik.\nCoordinates x, y, z are data indexes normalized in range [0,1]."), name, val))
 	{
 		mglVar *v = parser.AddVar(name.toAscii());
 		v->d = var->d.Momentum('y', val.toAscii());
@@ -671,7 +440,8 @@ void TableWindow::momenty()
 void TableWindow::momentz()
 {
 	QString name, val;
-	if(namesDialog(tr("UDAV - Momentum along 'z'"), tr("Specify which momentum evaluate.\nFor example, 'x2y' will give moment 'x^2*y'."), name, val))
+	if(namesDialog(tr("UDAV - Momentum along 'z'"),
+		tr("Specify which momentum evaluate.\nThe momentum is res_k = sum_ij how(x_i,y_j,z_k) a_ij/ sum_ij a_ij.\nCoordinates x, y, z are data indexes normalized in range [0,1]."), name, val))
 	{
 		mglVar *v = parser.AddVar(name.toAscii());
 		v->d = var->d.Momentum('z', val.toAscii());
@@ -687,8 +457,8 @@ void TableWindow::hist()
 	QPushButton *b;
 	QDialog *d = new QDialog(this);	d->setWindowTitle(tr("UDAV - Make histogram"));
 	QGridLayout *g = new QGridLayout(d);
-	l = new QLabel(tr("From value"), d);	g->addWidget(l,0,0);
-	l = new QLabel(tr("To value"), d);	g->addWidget(l,0,1);
+	l = new QLabel(tr("From"), d);	g->addWidget(l,0,0);
+	l = new QLabel(tr("To"), d);	g->addWidget(l,0,1);
 	v1 = new QLineEdit(d);	g->addWidget(v1,1,0);
 	v2 = new QLineEdit(d);	g->addWidget(v2,1,1);
 	l = new QLabel(tr("Number of points"), d);	g->addWidget(l,2,0);
